@@ -38,12 +38,14 @@
 #include <glib/gi18n.h>
 #include <dbus/dbus-glib.h>
 
+#include <libhal-gdevice.h>
+#include <libhal-gmanager.h>
+#include <libdbus-proxy.h>
+
 #include "gpm-brightness-lcd.h"
 #include "gpm-common.h"
 #include "gpm-debug.h"
-#include "gpm-hal.h"
 #include "gpm-marshal.h"
-#include "gpm-proxy.h"
 
 #define DIM_INTERVAL		10 /* ms */
 
@@ -59,8 +61,7 @@ struct GpmBrightnessLcdPrivate
 	guint			 level_std_hw;
 	guint			 levels;
 	gchar			*udi;
-	GpmProxy		*gproxy;
-	GpmHal			*hal;
+	DbusProxy		*gproxy;
 };
 
 enum {
@@ -69,7 +70,8 @@ enum {
 };
 
 G_DEFINE_TYPE (GpmBrightnessLcd, gpm_brightness_lcd, G_TYPE_OBJECT)
-static guint	     signals [LAST_SIGNAL] = { 0, };
+static guint signals [LAST_SIGNAL] = { 0, };
+static gpointer gpm_brightness_object = NULL;
 
 /**
  * gpm_brightness_lcd_get_hw:
@@ -91,11 +93,11 @@ gpm_brightness_lcd_get_hw (GpmBrightnessLcd *brightness,
 	g_return_val_if_fail (brightness != NULL, FALSE);
 	g_return_val_if_fail (GPM_IS_BRIGHTNESS_LCD (brightness), FALSE);
 
-	proxy = gpm_proxy_get_proxy (brightness->priv->gproxy);
+	proxy = dbus_proxy_get_proxy (brightness->priv->gproxy);
 	if (proxy == NULL) {
 		gpm_warning ("not connected to HAL");
 		return FALSE;
-	}	
+	}
 
 	ret = dbus_g_proxy_call (proxy, "GetBrightness", &error,
 				 G_TYPE_INVALID,
@@ -141,11 +143,11 @@ gpm_brightness_lcd_set_hw (GpmBrightnessLcd *brightness,
 	g_return_val_if_fail (brightness != NULL, FALSE);
 	g_return_val_if_fail (GPM_IS_BRIGHTNESS_LCD (brightness), FALSE);
 
-	proxy = gpm_proxy_get_proxy (brightness->priv->gproxy);
+	proxy = dbus_proxy_get_proxy (brightness->priv->gproxy);
 	if (proxy == NULL) {
 		gpm_warning ("not connected to HAL");
 		return FALSE;
-	}	
+	}
 
 	if (brightness_level_hw < 0 ||
 	    brightness_level_hw > brightness->priv->levels - 1) {
@@ -191,7 +193,6 @@ gpm_brightness_lcd_dim_hw_step (GpmBrightnessLcd *brightness,
 {
 	guint current_hw;
 	gint a;
-	gboolean ret;
 
 	g_return_val_if_fail (brightness != NULL, FALSE);
 	g_return_val_if_fail (GPM_IS_BRIGHTNESS_LCD (brightness), FALSE);
@@ -207,21 +208,13 @@ gpm_brightness_lcd_dim_hw_step (GpmBrightnessLcd *brightness,
 	if (new_level_hw > current_hw) {
 		/* going up */
 		for (a=current_hw; a <= new_level_hw; a+=step_interval) {
-			ret = gpm_brightness_lcd_set_hw (brightness, a);
-			/* we failed the last brightness set command, don't keep trying */
-			if (ret == FALSE) {
-				break;
-			}
+			gpm_brightness_lcd_set_hw (brightness, a);
 			g_usleep (1000 * DIM_INTERVAL);
 		}
 	} else {
 		/* going down */
 		for (a=current_hw; (gint) (a + 1) > (gint) new_level_hw; a-=step_interval) {
-			ret = gpm_brightness_lcd_set_hw (brightness, a);
-			/* we failed the last brightness set command, don't keep trying */
-			if (ret == FALSE) {
-				break;
-			}
+			gpm_brightness_lcd_set_hw (brightness, a);
 			g_usleep (1000 * DIM_INTERVAL);
 		}
 	}
@@ -511,11 +504,7 @@ gpm_brightness_lcd_finalize (GObject *object)
 	if (brightness->priv->gproxy != NULL) {
 		g_object_unref (brightness->priv->gproxy);
 	}
-	if (brightness->priv->hal != NULL) {
-		g_object_unref (brightness->priv->hal);
-	}
 
-	g_return_if_fail (brightness->priv != NULL);
 	G_OBJECT_CLASS (gpm_brightness_lcd_parent_class)->finalize (object);
 }
 
@@ -556,15 +545,16 @@ static void
 gpm_brightness_lcd_init (GpmBrightnessLcd *brightness)
 {
 	gchar **names;
-	gchar *manufacturer_string = NULL;
 	gboolean res;
+	HalGManager *manager;
+	HalGDevice *device;
 
 	brightness->priv = GPM_BRIGHTNESS_LCD_GET_PRIVATE (brightness);
 
-	brightness->priv->hal = gpm_hal_new ();
-
 	/* save udi of lcd adapter */
-	gpm_hal_device_find_capability (brightness->priv->hal, "laptop_panel", &names, NULL);
+	manager = hal_gmanager_new ();
+	hal_gmanager_find_capability (manager, "laptop_panel", &names, NULL);
+	g_object_unref (manager);
 	if (names == NULL || names[0] == NULL) {
 		gpm_warning ("No devices of capability laptop_panel");
 		return;
@@ -572,51 +562,39 @@ gpm_brightness_lcd_init (GpmBrightnessLcd *brightness)
 
 	/* We only want first laptop_panel object (should only be one) */
 	brightness->priv->udi = g_strdup (names[0]);
-	gpm_hal_free_capability (brightness->priv->hal, names);
+	hal_gmanager_free_capability (names);
 
+	/* FIXME: This should be a HAL property */
 	brightness->priv->does_own_dimming = FALSE;
 
-	/* If the manufacturer is IBM, then assume we are a ThinkPad,
-	 * and don't do the new-fangled dimming routine. The ThinkPad dims
-	 * gently itself and the two dimming routines just get messy.
-	 * https://bugzilla.redhat.com/bugzilla/show_bug.cgi?id=173382 */
-	gpm_hal_device_get_string (brightness->priv->hal, HAL_ROOT_COMPUTER,
-				   "smbios.system.manufacturer",
-				   &manufacturer_string, NULL);
-	if (manufacturer_string) {
-		/* FIXME: This should be a HAL property */
-		if (strcmp (manufacturer_string, "IBM") == 0) {
-			brightness->priv->does_own_dimming = TRUE;
-		}
-		g_free (manufacturer_string);
-	}
-
+	device = hal_gdevice_new ();
+	hal_gdevice_set_udi (device, brightness->priv->udi);
 	/* We only want to change the brightness if the machine does not
 	   do it on it's own updates, as this can make the panel flash in a
 	   feedback loop. */
-	res = gpm_hal_device_get_bool (brightness->priv->hal, brightness->priv->udi,
-				       "laptop_panel.brightness_in_hardware",
-				       &brightness->priv->does_own_updates, NULL);
+	res = hal_gdevice_get_bool (device, "laptop_panel.brightness_in_hardware",
+				    &brightness->priv->does_own_updates, NULL);
 	/* This key does not exist on normal machines */
-	if (!res) {
+	if (res == FALSE) {
 		brightness->priv->does_own_updates = FALSE;
 	}
 
-	/* get a managed proxy */
-	brightness->priv->gproxy = gpm_proxy_new ();
-	gpm_proxy_assign (brightness->priv->gproxy,
-			  GPM_PROXY_SYSTEM,
-			  HAL_DBUS_SERVICE,
-			  brightness->priv->udi,
-			  HAL_DBUS_INTERFACE_LAPTOP_PANEL);
-
 	/* get levels that the adapter supports -- this does not change ever */
-	gpm_hal_device_get_uint (brightness->priv->hal, brightness->priv->udi,
-				 "laptop_panel.num_levels", &brightness->priv->levels, NULL);
+	hal_gdevice_get_uint (device, "laptop_panel.num_levels",
+			      &brightness->priv->levels, NULL);
 	gpm_debug ("Laptop panel levels: %i", brightness->priv->levels);
 	if (brightness->priv->levels == 0 || brightness->priv->levels > 256) {
 		gpm_warning ("Laptop panel levels are invalid!");
 	}
+	g_object_unref (device);
+
+	/* get a managed proxy */
+	brightness->priv->gproxy = dbus_proxy_new ();
+	dbus_proxy_assign (brightness->priv->gproxy,
+			  DBUS_PROXY_SYSTEM,
+			  HAL_DBUS_SERVICE,
+			  brightness->priv->udi,
+			  HAL_DBUS_INTERFACE_LAPTOP_PANEL);
 
 	/* this changes under our feet */
 	gpm_brightness_lcd_get_hw (brightness, &brightness->priv->current_hw);
@@ -639,38 +617,40 @@ gpm_brightness_lcd_init (GpmBrightnessLcd *brightness)
 gboolean
 gpm_brightness_lcd_has_hw (void)
 {
-	GpmHal *hal;
+	HalGManager *manager;
 	gchar **names;
 	gboolean ret = TRUE;
 
 	/* okay, as singleton - so we don't allocate more memory */
-	hal = gpm_hal_new ();
-	gpm_hal_device_find_capability (hal, "laptop_panel", &names, NULL);
+	manager = hal_gmanager_new ();
+	hal_gmanager_find_capability (manager, "laptop_panel", &names, NULL);
+	g_object_unref (manager);
 
 	/* nothing found */
 	if (names == NULL || names[0] == NULL) {
 		ret = FALSE;
 	}
 
-	gpm_hal_free_capability (hal, names);
-	g_object_unref (hal);
+	hal_gmanager_free_capability (names);
 	return ret;
 }
 
 /**
  * gpm_brightness_lcd_new:
  * Return value: A new brightness class instance.
+ * Can return NULL if no suitable hardware is found.
  **/
 GpmBrightnessLcd *
 gpm_brightness_lcd_new (void)
 {
-	static GpmBrightnessLcd *brightness = NULL;
-	if (brightness != NULL) {
-		g_object_ref (brightness);
+	if (gpm_brightness_object != NULL) {
+		g_object_ref (gpm_brightness_object);
 	} else {
-		if (gpm_brightness_lcd_has_hw () == TRUE) {
-			brightness = g_object_new (GPM_TYPE_BRIGHTNESS_LCD, NULL);
+		if (gpm_brightness_lcd_has_hw () == FALSE) {
+			return NULL;
 		}
+		gpm_brightness_object = g_object_new (GPM_TYPE_BRIGHTNESS_LCD, NULL);
+		g_object_add_weak_pointer (gpm_brightness_object, &gpm_brightness_object);
 	}
-	return GPM_BRIGHTNESS_LCD (brightness);
+	return GPM_BRIGHTNESS_LCD (gpm_brightness_object);
 }
